searchData={"items":[{"type":"module","title":"EctoClone","doc":"Documentation for `EctoClone`.","ref":"EctoClone.html"},{"type":"function","title":"EctoClone.clone/5","doc":"Deep clones a target record.\n\n`target` is the record to be cloned.\n\n`repo` is the ecto repo to operate on, the current implementation supports only a single repo.\n\n`new_attrs` is a map that will override values on the clone of the target.\n\n`related_schemas_with_options` is the list of schemas to clone in relation to the target.\nThe order of schemas does not matter, EctoClone determines clone order by associations.\nSee examples for setting options per schema. Suppored options include:\n\n`map` - a function that takes the source row fields as a map and returns the cloned row fields\n\n`where` - a function that appends a where clause to the sql query and thus filters which rows should be cloned","ref":"EctoClone.html#clone/5"},{"type":"function","title":"Examples - EctoClone.clone/5","doc":"To clone a post with it's comments:\n    post = Repo.insert!(%Post{title: \"hello\"})\n    Repo.insert!(%Comment{body: \"p\", post_id: post.id})\n    {:ok, clone_id} = EctoClone.clone(post, Repo, %{title: \"New Title\"}, [Comment])\n\nTo clone a post with it's tags, comments, comment edits:\n    EctoClone.clone(post, Repo, %{}, [\n      PostTag,\n      Comment,\n      CommentEdit\n    ])\n\nOptions on schemas allow chaning values of cloned records:\n    EctoClone.clone(post, Repo, %{}, [\n      PostTag,\n      [Comment, map: fn comment -> Map.put(comment, :likes, 0) end],\n      CommentEdit\n    ])\n\nThe where option allows selective cloning at the schema level:\n    EctoClone.clone(post, Repo, %{}, [\n      PostTag,\n      [Comment, where: fn query -> where(query, [comment], comment.likes >= 0) end],\n      CommentEdit\n    ])\n\nIf you use the ecto timestamps and want to set inserted_at on all of your cloned records, you can use the map option globally:\n    now = your_get_timestamp_function()\n    EctoClone.clone(\n      post,\n      Repo,\n      %{},\n      [PostTag, Comment, CommentEdit],\n      map: fn r -> Map.put(r, :inserted_at, now) end\n    )","ref":"EctoClone.html#clone/5-examples"},{"type":"function","title":"Association Requirements - EctoClone.clone/5","doc":"Each related schema being cloned must have either a belongs_to or has_one through association to the target.\n\nA Comment can be cloned along with a Post because it has `belongs_to :post, Post`\n\nA CommentEdit can be cloned with a Post because it has `belongs_to :comment, Comment` and `has_one :post, through: [:comment, :post]`\n\nIt is possible to have multiple possible relations to the target. A schema can have multiple has_one through:\n    schema \"comment_pair\" do\n      belongs_to :comment_a, Comment\n      belongs_to :comment_b, Comment\n\n      has_one :post_a, through: [:comment_a, :post]\n      has_one :post_b, through: [:comment_b, :post]\n    end\n\nOr a schema can have both a belongs_to AND has_one through:\n    schema \"moderation_flag\" do\n      belongs_to :post, Post\n      belongs_to :comment, Comment\n\n      has_one :comment_post, through: [:comment, :post]\n    end\n\nThese examples, while contrived, show that a record can have multple potential paths to being related to the target.\nEvery possible path is exhausted when determing if a record is related to the clone target.","ref":"EctoClone.html#clone/5-association-requirements"},{"type":"type","title":"EctoClone.related_schema_with_options/0","doc":"","ref":"EctoClone.html#t:related_schema_with_options/0"},{"type":"type","title":"EctoClone.target/0","doc":"","ref":"EctoClone.html#t:target/0"},{"type":"extras","title":"EctoClone","doc":"# EctoClone\n\n[![Build Status](https://github.com/grantwest/ecto_clone/actions/workflows/ci.yml/badge.svg)](https://github.com/grantwest/ecto_clone/actions/workflows/ci.yml)\n[![Version](https://img.shields.io/hexpm/v/ecto_clone.svg)](https://hex.pm/packages/ecto_clone)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecto_clone/)\n[![Download](https://img.shields.io/hexpm/dt/ecto_clone.svg)](https://hex.pm/packages/ecto_clone)\n[![License](https://img.shields.io/badge/License-0BSD-blue.svg)](https://opensource.org/licenses/0bsd)\n[![Last Updated](https://img.shields.io/github/last-commit/grantwest/ecto_clone.svg)](https://github.com/grantwest/ecto_clone/commits/master)\n\nTake advantage of Ecto associations to deep clone data in your database.\n\nTo clone a post with it's comments and tags:\n\n```elixir\n{:ok, cloned_post_id} = EctoClone.clone(%Post{id: 5}, Repo, %{title: \"new title\"}, [Comment, PostTag])\n```\n\nSee [clone docs](https://hexdocs.pm/ecto_clone/EctoClone.html#clone/4) for more information and examples.\n\n#","ref":"readme.html"},{"type":"extras","title":"Todo - EctoClone","doc":"- [ ] error when intermediate schemas are missing\n- [ ] support mysql\n- [ ] support sqlite\n- [ ] clone tables in parallel\n- [ ] fix self references in parallel\n- [ ] support foreign key != :id\n- [ ] error when circular associations","ref":"readme.html#todo"}],"content_type":"text/markdown"}